@@ -1,5 +1,22 @@
+# daterangeparser - a Python library to parse string date ranges
+# Copyright (C) 2013  Robin Wilson
+
+# This program is free software: you can redistribute it and/or modify
+# it under the terms of the GNU Lesser General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU Lesser General Public License
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+
 from pyparsing import *
 import datetime
+import calendar
 
 def check_day(tokens):
   """Converts to int and checks a day number, ensuring it is > 1 and < 31.
@@ -67,6 +84,8 @@ def post_process(res):
     # We have a single date, not a range
     res['start'] = {}
     res['start']['day'] = res.end.day
+    if res['start']['day'] == '':
+      res['start']['day'] = 1
     res['start']['month'] = res.end.month
     if 'year' not in res.end:
       res['start']['year'] = today.year
@@ -86,6 +105,12 @@ def post_process(res):
   # Sort out months
   if 'month' not in res.start:
     res.start['month'] = res.end.month
+
+  if 'day' not in res.start or res.start['day'] == '':
+    res.start['day'] = 1
+
+  if 'day' not in res.end or res.end['day'] == '':
+    res.end['day'] = calendar.monthrange(res.end.year, res.end.month)[1]
   
   return res
 
@@ -116,10 +141,10 @@ def create_parser():
   time = hours("hour") + time_sep.suppress() + mins("mins") + Optional(am_pm)("meridian")
   
   # Starting date
-  first_date = Group(Optional(time).suppress() & Optional(day).suppress() & full_day_string("day") & Optional(month("month")) & Optional(year("year")))
+  first_date = Group(Optional(time).suppress() & Optional(day).suppress() & Optional(full_day_string("day")) & Optional(month("month")) & Optional(year("year")))
   
   # Ending date
-  last_date = Group(Optional(time).suppress() & Optional(day).suppress() & full_day_string("day") & month("month") & Optional(year("year")))
+  last_date = Group(Optional(time).suppress() & Optional(day).suppress() & Optional(full_day_string("day")) & month("month") & Optional(year("year")))
   
   # Possible separators
   separator = oneOf(u"- -- to until \u2013 \u2014 ->", caseless=True)
@@ -128,7 +153,7 @@ def create_parser():
   ignoreable_chars = oneOf(", from", caseless=True)
   
   # Final putting together of everything
-  daterange = Optional(first_date("start") + separator.suppress()) + last_date("end") + stringEnd
+  daterange = Optional(first_date("start") + Optional(time).suppress() + separator.suppress()) + last_date("end") + Optional(time).suppress() + stringEnd
   daterange.ignore(ignoreable_chars)
 
   return daterange
@@ -174,6 +199,7 @@ def parse(text):
   """
   parser = create_parser()
   
+  print text
   result = parser.parseString(text)
   #print result.dump()
   #print "----------"
